/* eslint-disable @typescript-eslint/interface-name-prefix */
export type TOAST_SERVICE_ZONE = 'ALPHA' | 'BETA' | 'REAL';
export type MARKET_ID = 'HGSTORE' | 'ONGATE';
export type IAP_POINT_CURRENCY = 'HAN';
export type TOAST_LOGGER_LEVEL = 'DEBUG' | 'INFO' | 'WARN' | 'ERROR' | 'FATAL';

/**
 * Iap point
 */
export class IapPoint {
  private static readonly SERVICE_NAME;
  private static toastIapPoints;
  private static instance;
  private static token;
  private static userId;
  /**
   * Initializes iap point
   * @param configuration
   */
  static initialize(configuration: ToastIapPointConfiguration): void;
  /**
   * Sets token
   * @param token
   * "HG" : Gamebase Token
   */
  static setToken(token: string): void;
  /**
   * Sets user id
   * @param userId
   * "HG" : Gamebase UserId
   * "ONGATE" : Ongate UserId
   */
  static setUserId(userId: string): void;
  /**
   * Sets debug mode
   * @param debugMode
   */
  static setDebugMode(debugMode: boolean): void;

  /**
   * Request products details
   * @returns
   */
  static requestProductDetails(): ToastPromise<IAPPointProduct[]>;

  /**
   * Request consumable purchases
   * @returns
   */
  static requestConsumablePurchases(): ToastPromise<IAPPointPurchase[]>;

  /**
   * Inquirys balance
   * @returns
   */
  static inquiryBalance(): ToastPromise<IAPPointBalance>;
  /**
   * Charges iap point
   * @returns
   */
  static charge(): ToastPromise<IAPPointBalance>;
  /**
   * Purchases iap point
   * @param productId
   * @param payload
   * @returns
   */
  static purchase(
    productId: string,
    payload?: {
      developerPayload?: string;
      gamebasePayload?: string;
      toastPayload?: string;
    }
  ): ToastPromise<IAPPointPurchase>;
  /**
   * Gets iap point count
   * @returns
   */
  private static getIapPointCount;
  /**
   * Determines whether not initialize is
   * @returns
   */
  static isNotInitialize(): boolean;
  /**
   * Determines whether first initialize is
   * @returns
   */
  static isFirstInitialize(): boolean;
}

/**
 * Toast iap point configuration
 */
export interface ToastIapPointConfiguration {
  storeCode: MARKET_ID;
  appKey: string;
  serviceZone: TOAST_SERVICE_ZONE;
}

/**
 * Toast iap point product detail information
 */
export interface IAPPointProduct {
  productId: string;
  productSeq: number;
  productName: string;
  productType: string;
  price: number;
  currency: string;
  regYmdt: string;
  usingStatus: string;
}

/**
 * Toast iap point balace
 */
export interface IAPPointBalance {
  token: string;
  balance: number;
}

/**
 * Toast iap point purchase receipt
 */
export interface IAPPointPurchase {
  userId: string;
  productId: string;
  paymentSeq: string;
  price: number;
  currency: string;
  paymentId: string;
  accessToken: string;
  purchaseTimeMills?: number;
  developerPayload?: string;
  gamebasePayload?: string;
  toastPayload?: string;
}

/**
 * Logger
 */
export class Logger {
  private static readonly SERVICE_NAME;
  private static toastLoggers;
  private static instance;
  /**
   * Initializes logger
   * @param configuration
   * @returns initialize
   */
  static initialize(configuration: ToastLoggerConfiguration): ToastLogger;
  /**
   * Debugs logger
   * @param message
   * @param [userFields]
   * @returns
   */
  static debug(message: string, userFields?: Dictionary<string>): ToastPromise;
  /**
   * Infos logger
   * @param message
   * @param [userFields]
   * @returns
   */
  static info(message: string, userFields?: Dictionary<string>): ToastPromise;
  /**
   * Warns logger
   * @param message
   * @param [userFields]
   * @returns
   */
  static warn(message: string, userFields?: Dictionary<string>): ToastPromise;
  /**
   * Fatals logger
   * @param message
   * @param [userFields]
   * @returns
   */
  static fatal(message: string, userFields?: Dictionary<string>): ToastPromise;
  /**
   * Errors logger
   * @param message
   * @param [userFields]
   * @returns
   */
  static error(message: string, userFields?: Dictionary<string>): ToastPromise;
  /**
   * Sets user field
   * @param key
   * @param value
   * @returns
   */
  static setUserField(key: string, value: string): void;
  /**
   * Reports logger
   * @param logLevel
   * @param message
   * @param [userFields]
   * @returns
   */
  static report(
    logLevel: TOAST_LOGGER_LEVEL,
    message: string,
    userFields?: Dictionary<string>
  ): ToastPromise;
  /**
   * Gets loggers count
   * @returns
   */
  private static getLoggersCount;
  /**
   * Determines whether not initialize is
   * @returns
   */
  static isNotInitialize(): boolean;
  /**
   * Determines whether first initialize is
   * @returns
   */
  static isFirstInitialize(): boolean;
}

/**
 * Toast logger configuration
 */
export interface ToastLoggerConfiguration {
  appKey: string;
  serviceZone: TOAST_SERVICE_ZONE;
  projectVersion: string;
  useWebWorker?: boolean;
}

/**
 * Toast logger
 */
export class ToastLogger {
  initialize(configuration: ToastLoggerConfiguration): boolean;
  log(logLevel: TOAST_LOGGER_LEVEL, message: string, userFields: Dictionary<string>): ToastPromise;
  report(
    logLevel: TOAST_LOGGER_LEVEL,
    message: string,
    userFields: Dictionary<string>
  ): ToastPromise;
  setUserField(key: string, value: string): void;
  debug(message: string, userFields?: Dictionary<string>): ToastPromise;
  info(message: string, userFields?: Dictionary<string>): ToastPromise;
  warn(message: string, userFields?: Dictionary<string>): ToastPromise;
  fatal(message: string, userFields?: Dictionary<string>): ToastPromise;
  error(message: string, userFields?: Dictionary<string>): ToastPromise;
}

/**
 * Dictionary
 * @template T
 */
export interface Dictionary<T> {
  [index: string]: T;
}

/**
 * Toast response
 * @template T
 */
export class ToastResponse<T = any> {
  data?: T;
  result: ToastResult;
  constructor(result: ToastResult, data?: T);
}
/**
 * Toast result
 */
export class ToastResult {
  isSuccessful: boolean;
  resultCode: number;
  resultString: string;
  constructor(isSuccessful: boolean, resultCode: number, resultString: string);
}
/**
 * Toast promise
 * @template T
 */
export class ToastPromise<T = any> extends Promise<ToastResponse<T>> {}

declare const _default: {
  IapPoint: typeof IapPoint;
  Logger: typeof Logger;
};
export { _default };
